require 'parseconfig'
require 'cocoapods'
require 'pathname'
require_relative "module.rb"
require_relative "podspecs.rb"

module Qricker

  class DependencyNode
    attr_accessor :name
    attr_accessor :childs
    def initialize(name)
      @name = name
      @childs = []
    end

    def hasChild(childName)
      subs = @childs.map { |c|
        c if  c.name == childName
      }.compact
      subs.count > 0
    end

    def removeChild(child)
      @childs.delete child
    end

    def containsChild?(child)
      not searchChild(child).nil?
    end

    def addChild(child)
      if containsChild? child.name
      else
        @childs << child
      end
    end

    def searchChild(childName)
      aimChild = nil
      for child in @childs
        if (child.name == childName)
          aimChild = child
          break
        else
          aimChild = child.searchChild(childName)
          if not aimChild.nil?
            break
          end
        end
      end

      return aimChild
    end

    def printSelf(level)
      line = "--"
      level.times {
        line+="--"
      }
      puts "#{line}#{@name}"
      @childs.each { |child|
        child.printSelf(level+1)
      }
    end

  end
  class Resolver
    def initialize(deps)
      @roots = DependencyNode.new("root")
      @deps = deps
      deps.each { |spec|
        dependencies =  spec.podSpecilication.attributes_hash["dependencies"]
        next if  dependencies.nil?
        dependencies.each { |name , versions|
          addDependency(name, spec.name)
        }
      }
    end

    def addDependency(parent, child)
      parentNode = @roots.searchChild(parent)
      if parentNode.nil?
        parentNode = DependencyNode.new(parent)
        @roots.addChild(parentNode)
      end
      childNode = nil
      if @roots.hasChild(child)
        childNode = @roots.searchChild(child)
        @roots.removeChild childNode
      else
        childNode = DependencyNode.new(child)
      end
      if parentNode.searchChild(child).nil?
        parentNode.addChild(childNode)
      end
    end

    def sorted

      array = Array.new
      addSortedArray(array, @roots)
      deps = @deps.sort{|x, y|
        xIndex = array.index(x.name)
        yIndex =  array.index(y.name)
        xIndex <=> yIndex
      }

      return deps
    end


    def printSelf
      @roots.printSelf(0)
    end

    private
    def addSortedArray(array, node)
      baseIndex = array.count
      for n in node.childs
        if array.include?(n.name)
          index = array.index(n.name)
          baseIndex = index+1
        else
          array.insert(baseIndex, n.name)
        end
      end

      for n in node.childs
         addSortedArray(array, n)
      end
    end
  end

  def Qricker.collectDependencies(config,dependecies, originSpec)
    if config.get_groups.include?(originSpec)
      podspec = Pod::Specification.from_file(config[originSpec]["PATH"]+"/#{originSpec}.podspec")
      podspec.dependencies.each { |pspec|
        if not dependecies.include? pspec.name
          dependecies << pspec.name
          collectDependencies(config,dependecies, pspec.name)
        end
      }
    end
  end


  def Qricker.dependency(localfile , specnames)
    config = ParseConfig.new(localfile)
    if config.nil?
      return []
    end
    allDependencies = []
    if not specnames.nil?
      specnames.each { |name|
        collectDependencies(config,allDependencies, name)
      }
    end

    outputDependencies = []
    allDependencies.each { |dep|
      if config.get_groups.include?(dep)
        spec = RickerSpec.new(dep, config[dep]["PATH"])
        outputDependencies << spec
      end
    }
    return sortDenpencies outputDependencies
  end

  def Qricker.sortDenpencies(deps)
    resolver = Resolver.new(deps)
    deps = resolver.sorted

    puts "依赖关系图为："
    resolver.printSelf
    return deps
  end

  def Qricker.dependencyIncludeSelf(localfile, specnames)
    config = ParseConfig.new(localfile)
    if config.nil?
      return []
    end
    allDependencies = Array.new(specnames)
    if not specnames.nil?
      specnames.each { |name|
        collectDependencies(config,allDependencies, name)
      }
    end

    outputDependencies = []
    allDependencies.each { |dep|
      if config.get_groups.include?(dep)
        spec = RickerSpec.new(dep, config[dep]["PATH"])
        outputDependencies << spec
      end
    }
    return sortDenpencies(outputDependencies)
  end
end
