# -*- coding: UTF-8 -*-

require_relative "module_cache"
module Qricker

  class SpecBuilder
    def initialize(spec, moduleCache)
      @spec = spec
      @moduleCache = moduleCache
    end

    def framework_path
      if @embedded
        @spec.name + '.embeddedframework' + '/' + @spec.name + '.framework'
      else
        @spec.name + '.framework'
      end
    end

    def spec_platform(platform)
      fwk_base = platform.name.to_s + '/' + framework_path
      spec = <<RB
  s.#{platform.name}.deployment_target    = '#{platform.deployment_target}'
  s.#{platform.name}.vendored_framework   = '#{fwk_base}'
RB

      %w(frameworks weak_frameworks libraries requires_arc xcconfig).each do |attribute|
        attributes_hash = @spec.attributes_hash[platform.name.to_s]
        next if attributes_hash.nil?
        value = attributes_hash[attribute]
        next if value.nil?

        value = "'#{value}'" if value.class == String
        spec += "  s.#{platform.name}.#{attribute} = #{value}\n"
      end
      spec
    end

    def spec_metadata
      spec = spec_header
      spec += spec_dependencies
      spec
    end

    def spec_close
      "end\n"
    end



    def spec_dependencies
      value = @spec.attributes_hash["dependencies"]
      return "" if value.nil?
      return "" if value.class == Array and value.length == 0
      spec = ""
      value.each { |dep|
        spec += "\n  s.dependency '#{dep[0]}' "
        if not @moduleCache.nil?
          depSpec = @moduleCache.moduleByName(dep[0])
          if not depSpec.nil? and  not depSpec.podSpecilication.nil?
             spec += ", '#{depSpec.podSpecilication.version}'"
          end
        else
          dep[1].each { |req|
           spec += ", '#{req}'"
          }
        end

      }
      spec
    end

    def spec_origin_sources
      value = @spec.attributes_hash["source_files"]
      return "" if value.nil?
      "  s.source_files = #{transferValueToS(value)}\n"
    end

    def transferValueToS(value)
      return "" if value.nil?
      valueStr = ""
      if value.class == String
        value = value.dump
        valueStr = "#{value}\n"
      elsif value.class == Array
        value.each_with_index {|each , index|
          if index != 0
            valueStr += ", "
          end
          valueStr += "'#{each}'"
        }
      elsif value.class == Hash
        valueStr = "#{value}"
      end
      return valueStr
    end

    private
    def spec_header
      spec = "Pod::Spec.new do |s|\n"

      %w(name version summary license authors homepage description social_media_url
         docset_url documentation_url screenshots frameworks weak_frameworks libraries requires_arc
         deployment_target xcconfig prefix_header_file module_name prepare_command platforms ).each do |attribute|
        value = @spec.attributes_hash[attribute]
        next if value.nil?
        spec += "  s.#{attribute} = #{transferValueToS(value)}\n"
      end
      spec
    end
  end

end